#
#    Copyright (C) 2003-2004  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
This module abstracts the access to file contents and file attributes.

Alternative backend data stores such as databases or version control systems
could be implemented by implementing an identical interface.

This implementation works by storing each "file" as a directory with the
extension ".ldv". The directory may then contain not only the content
(potentially in multiple forms) but additional metadata, annotations, etc.
"""

import os        
        
        
class LeonardoFileSystem:
        

    def __init__(self, root):
        self.root = root

        # check access to filesystem root
        if not os.access(self.root, os.R_OK):
            # @@@ improve response to user
            raise "can't access filesystem root"
        
        
    # @@@ at present you are required to specify the content type to
    # @@@ determine the extension to use for the content file.
    # @@@ it would be nice if this were optional but it's not clear
    # @@@ how ambiguities should be handled.
    def get(self, key, content_type):
        return LeonardoFile(self, key, content_type)


    def get_children(self, key):
        """
        Returns a tuple of:
          - list of directories that are children of the key
          - list of leonardo pages that are children of the key
        """

        directories = []
        files = []
    
        # @@@ factor out commonalities with LeonardoFile
        d = self.root + key.strip("/")

        if os.path.isdir(d) and not d.endswith(".ldv"):
            for f in os.listdir(d):
                # @@@ factor out ignore list
                if os.path.isdir(d + "/" + f) and f not in [".svn"]:
                    if f.endswith(".ldv"):
                        files.append(f[:-4])
                    else:
                        directories.append(f)
        directories.sort()
        directories.reverse()
        files.sort()
        files.reverse()
            
        return directories, files



class LeonardoFile:

    
    def __init__(self, filesystem, key, content_type):
        self.filesystem = filesystem
        self.key = key
        self.content_type = content_type
        self.content = None


    # @@@ refactor to filesystem?
    def __get_directory(self):
        return self.filesystem.root + self.key.strip("/") + ".ldv/"


    # @@@ refactor to filesystem?
    def __get_content_filename(self):
        return self.__get_directory() + "__content__." + self.content_type


    def exists(self):
        try:
            f = file(self.__get_content_filename())
            f.close()
            return True
        except IOError:
            return False
        

    # @@@ make private?
    def load_content(self):
        f = file(self.__get_content_filename())    
        s = f.read()
        f.close()
        self.content = s


    def get_content(self):
        if not self.content:
            self.load_content()
        return self.content        


    def get_lastmod(self):
        import os
        ST_MTIME = 8
        return os.stat(self.__get_content_filename())[ST_MTIME]


    def set_content(self, content):
        path = self.__get_content_filename()
        directory = path[:path.rfind("/")]
        try:
            d = file(directory)
            d.close()
        except:
            import os
            try:
                os.makedirs(directory)
            except:
                pass
        f = file(path, "w")
        f.write(content)
        f.close()
